<?php
/*
 #########################################################################
 #                       xt:Commerce VEYTON 4.0 Enterprise
 # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 #
 # Copyright 2007-2008 xt:Commerce GmbH. All Rights Reserved.
 # This file may not be redistributed in whole or significant part.
 # Content of this file is Protected By International Copyright Laws.
 #
 # ~~~~~~~~~ xt:Commerce VEYTON 4.0 Enterprise IS NOT FREE SOFTWARE ~~~~~~~~~~~~~~~~
 #
 # http://www.xt-commerce.com
 #
 # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 #
 # @version $Id: class.validation.vat_id.php 3174 2009-03-12 07:15:45Z matthias $
 # @copyright xt:Commerce GmbH, www.xt-commerce.com
 #
 # @author Mario Zanier, xt:Commerce GmbH	mzanier@xt-commerce.com
 #
 # @author Matthias Hinsche					mh@xt-commerce.com
 # @author Matthias Benkwitz				mb@xt-commerce.com
 #
 # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 #
 # xt:Commerce GmbH, Bachweg 1, A-6091 Goetzens (AUSTRIA)
 # office@xt-commerce.com
 #
 #########################################################################
 */

defined('_VALID_CALL') or die('Direct Access is not allowed.');

/**
 * vat id validation for EU countries
 * validation rules: http://www.pruefziffernberechnung.de/U/USt-IdNr.shtml#PZDK
 *
 */
class vat_id {
	var $vatid,$country_code;

	function vat_id() {

		$this->rules = array();
		$this->rules[] = 'BE';
		$this->rules[] = 'DK';
		$this->rules[] = 'DE';
		$this->rules[] = 'FI';

		$this->rules[] = 'EL';

		$this->rules[] = 'IT';
		$this->rules[] = 'LU';
		$this->rules[] = 'NL';
		$this->rules[] = 'AT';
		$this->rules[] = 'PL';
		$this->rules[] = 'PT';
		$this->rules[] = 'SE';
		$this->rules[] = 'SL';
	


		$this->liveCheck = array();
		$this->liveCheck[] = 'AD';
		$this->liveCheck[] = 'BG';
		$this->liveCheck[] = 'EE';
		$this->liveCheck[] = 'GI';
		$this->liveCheck[] = 'IS';
		$this->liveCheck[] = 'LV';
		$this->liveCheck[] = 'LT';
		$this->liveCheck[] = 'MT';
		$this->liveCheck[] = 'NO';
		$this->liveCheck[] = 'RO';
		$this->liveCheck[] = 'SK';
		$this->liveCheck[] = 'CZ';
		$this->liveCheck[] = 'HU';
		$this->liveCheck[] = 'CY';

		$this->liveCheck[] = 'FR';
		$this->liveCheck[] = 'GB';
		$this->liveCheck[] = 'IE';
		$this->liveCheck[] = 'ES';

	}

	/**
	 * check european vat id
	 * 
	 * check by algorithm for known vat id country algorithm, perform live check for unknown 
	 *
	 * @param string $vat_id
	 * @param string $country_code
	 * @return true/false , -2 for no check, -99 for live check servicefault
	 */
	function _check($vat_id,$country_code) {
		$this->country_code = $country_code;
        $this->country_code = strtoupper($this->country_code);

		$param ='/[^a-zA-Z0-9]/';
		$vat_id=preg_replace($param,'',$vat_id);

		$this->vatid = $vat_id;

		$this->vatid = strtoupper($this->vatid);

		// add iso code to vat id
		if (substr($this->vatid, 0, 2) != $this->country_code) {
			$this->vatid = $this->country_code.$this->vatid;
		}

		if (!in_array($this->country_code,$this->rules)) {
			// no check available
			if (in_array($this->country_code,$this->liveCheck)) {
				if (_STORE_VAT_CHECK_LIVE=='false') {
					// live check deactivated
					return -3;
				} else {
					return $this->_LIVE();
				}
			} else {
				// no live check available
				return -2;
			}
		} else {


			$function = '_'.strtoupper($this->country_code);
			$result = $this->$function();
			return $result;
		}

	}



	function _BE() {

		if (!$this->_checkLength(11)) return false;

		$checksumme = substr($this->vatid,2,7);

		$checksumme = $checksumme % 97;
		$checksumme = 97 - $checksumme;

		$check = (int) substr($this->vatid,-2);
		if ($check != $checksumme) return false;
		return true;

	}

	function _DK() {

		if (!$this->_checkLength(10)) return false;

		$weight = array(2,7,6,5,4,3,2,1);
		$params = array(3,4,5,6,7,8,9,10);
		$checksumme = $this->_calcWeightedSum($weight,$params,'summe');

		if ($checksumme % 11 > 0) return false;
		return true;

	}

	function _DE() {
		if (!$this->_checkLength(11)) return false;

		$m = 10;
		$n = 11;

		$produkt = $m;
		for ($i = 2; $i < 10; $i ++) {
			$summe = ($this->vatid[$i]+$produkt)%$m;
			if ($summe==0) $summe = $m;
			$produkt = (2*$summe) % $n;
		}
		$checksumme = $n-$produkt;
		if (!$this->_checkSingle(11,$checksumme)) return false;
		return true;

	}

	function _FI() {
		if (!$this->_checkLength(10)) return false;

		$weight = array(7,9,10,5,8,4,2);
		$params = array(3,4,5,6,7,8,9);
		$checksumme = $this->_calcWeightedSum($weight,$params,'summe');

		$checksumme = $checksumme % 11;
		$checksumme = 11 - $checksumme;
		if (!$this->_checkSingle(10,$checksumme)) return false;
		return true;

	}

	function _EL() {
		if (!$this->_checkLength(11)) return false;

		$weight = array(256,128,64,32,16,8,4,2);
		$params = array(3,4,5,6,7,8,9,10);
		$checksumme = $this->_calcWeightedSum($weight,$params,'summe');
		$checksumme = $checksumme % 11;
		if ($checksumme>9) $checksumme = 0;
		if (!$this->_checkSingle(11,$checksumme)) return false;
		return true;
	}


	function _IT() {
		if (!$this->_checkLength(13)) return false;

		$weight = array(1,2,1,2,1,2,1,2,1,2);
		$params = array(3,4,5,6,7,8,9,10,11,12);
		$checksumme = $this->_calcWeightedSum($weight,$params,'quersumme');
		$checksumme = $checksumme % 10;
		$checksumme = 10 - $checksumme;
		if ($checksumme==10) $checksumme = 0;
		if (!$this->_checkSingle(13,$checksumme)) return false;
		return true;
	}

	function _LU() {
		if (!$this->_checkLength(10)) return false;

		$checksumme = substr($this->vatid,2,6);
		$checksumme = $checksumme % 89;


		$check = (int) substr($this->vatid,-2);
		if ($check != $checksumme) return false;
		return true;

	}

	function _NL() {
		if (!$this->_checkLength(14)) return false;

		if (!$this->_checkSingle(12,'B')) return false;

		$weight = array(9,8,7,6,5,4,3,2);
		$params = array(3,4,5,6,7,8,9,10);
		$checksumme = $this->_calcWeightedSum($weight,$params,'summe');
		$checksumme = $checksumme % 11;
		if (!$this->_checkSingle(11,$checksumme)) return false;
		return true;

	}


	function _AT() {

		if (!$this->_checkLength(11)) return false;
		if (!$this->_checkSingle(3,'U')) return false;

		$weight = array(1,2,1,2,1,2,1);
		$params = array(4,5,6,7,8,9,10);
		$checksumme = $this->_calcWeightedSum($weight,$params,'quersumme');

		$checksumme = 96 - $checksumme;
		if ($checksumme>10) {
			$checksumme = (string)$checksumme;
			$checksumme = $checksumme[1];
		}
		if (!$this->_checkSingle(11,$checksumme)) return false;
		return true;

	}

	function _PL() {

		if (!$this->_checkLength(12)) return false;


		$weight = array(6,5,7,2,3,4,5,6,7);
		$params = array(3,4,5,6,7,8,9,10,11);
		$checksumme = $this->_calcWeightedSum($weight,$params,'summe');
		$checksumme = $checksumme % 11;

		if (!$this->_checkSingle(12,$checksumme)) return false;
		return true;

	}

	function _PT() {
		if (!$this->_checkLength(11)) return false;

		$weight = array(9,8,7,6,5,4,3,2);
		$params = array(3,4,5,6,7,8,9,10);
		$checksumme = $this->_calcWeightedSum($weight,$params,'summe');
		$checksumme = $checksumme % 11;
		$checksumme = 11-$checksumme;
		if ($checksumme>9) $checksumme=0;
		if (!$this->_checkSingle(11,$checksumme)) return false;
		return true;

	}

	function _SE() {
		if (!$this->_checkLength(14)) return false;

		$weight = array(2,1,2,1,2,1,2,1,2);
		$params = array(3,4,5,6,7,8,9,10,11);
		$checksumme = $this->_calcWeightedSum($weight,$params,'quersumme');
		$checksumme = $checksumme % 10;
		$checksumme = 10-$checksumme;

		if ($checksumme==10) $checksumme=0;
		if (!$this->_checkSingle(12,$checksumme)) return false;
		return true;

	}
	function _SL() {
		if (!$this->_checkLength(10)) return false;

		$weight = array(8,7,6,5,4,3,2);
		$params = array(3,4,5,6,7,8,9);
		$checksumme = $this->_calcWeightedSum($weight,$params,'summe');
		$checksumme = $checksumme % 11;
		$checksumme = 11 - $checksumme;
		if ($checksumme==0) return false;
		if ($checksumme==1) $checksumme = 0;
		if (!$this->_checkSingle(10,$checksumme)) return false;
		return true;

	}

	function _calcWeightedSum($weight,$params,$type = 'quersumme') {

		$sum = 0;
		foreach ($params as $key => $var) {
			switch ($type) {
				case 'quersumme':
					$wert = $this->vatid[$var-1];
					$multiplikator = $weight[$key];
					$wert = $wert*$multiplikator;
					if ($wert>=10) {
						$wert = (string)$wert;
						$wert = $wert[0]+$wert[1];
					}
					$sum += $wert;
					break;
				case 'summe':
					$wert = $this->vatid[$var-1];
					$multiplikator = $weight[$key];
					$wert = $wert*$multiplikator;
					$sum += $wert;
					break;
			}
		}
		return $sum;

	}

	/**
	 * check if string is in correct length
	 *
	 * @param int $length
	 * @return boolean
	 */
	function _checkLength($length) {
		if (strlen($this->vatid)!=$length) return false;
		return true;
	}

	/**
	 * check if single char matches
	 *
	 * @param int $pos
	 * @param string $char
	 * @return boolean
	 */
	function _checkSingle($pos,$char) {
		if($this->vatid[$pos-1]!=$char) return false;
		return true;
	}
	
	

	/**
	 * perform vat id LIVE check at VIES VAT number validation
	 *
	 * @return boolean
	 */
	function _LIVE() {
		global $logHandler;
		
		require_once 'xtFramework/library/nusoap/nusoap.php';

		$vat_id = str_replace($this->country_code,'',$this->vatid);
		$params = array('countryCode'=>$this->country_code,'vatNumber'=>$vat_id);
		$client = new nusoap_client('http://ec.europa.eu/taxation_customs/vies/api/checkVatPort');

		$result = $client->call('checkVat',$params);

		if (!is_array($result)) {
			// save system log
			$log_data = array();
			$log_data['error_str'] = $client->error_str;
			$log_data['debug_str'] = $client->debug_str;
			$log_data['endpoint'] = $client->endpoint;
			$logHandler->_addLog('error','vatid_check','0',$log_data);
			
			return -99;
		}
		if ($result['valid']=='1') return true;
		return false; 

	}

}
?>